package sedplugin.errors;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JFrame;

public class ErrorsLog {
	private static ErrorsLog currentInstance = null;

	private Vector<SEDException> exceptions;
	private Vector<SEDException> lastExceptions;

	private ErrorsNotifier notifier;

	private ErrorReport fraErrorsReport = null;

	public ErrorsLog(){
		this(null);
	}

	public ErrorsLog(ErrorsNotifier n){
		exceptions = new Vector<SEDException>();
		lastExceptions = new Vector<SEDException>();
		notifier = n;
	}

	public final static void setCurrentInstance(ErrorsLog newLog){
		currentInstance = newLog;
	}

	public final static ErrorsLog getCurrentInstance(){
		if (currentInstance == null)
			currentInstance = new ErrorsLog(null);

		return currentInstance;
	}

	public final void setNotifier(ErrorsNotifier n){
		notifier = n;
	}

	public final int getNbLastErrors(){ return lastExceptions.size(); }

	public final int getNbErrors(){ return exceptions.size(); }

	public final void addRefreshException(SEDException ex){
		addException(new RefreshException(ex));
	}

	public final void addException(SEDException ex){
		if (ex != null){
			exceptions.add(0, ex);
			lastExceptions.add(0, ex);
			if (notifier != null){
				notifier.updateCount();
				notifier.notifyUser(ex);
			}
		}
	}

	public final Iterator<SEDException> getLastErrors(){
		return lastExceptions.iterator();
	}

	public final Iterator<SEDException> getErrors(){
		return exceptions.iterator();
	}

	public final void clearLastExceptions(){
		lastExceptions.clear();
		notifier.updateCount();
	}

	public final void clear(){
		clearLastExceptions();
		notifier = null;
	}

	public final void displayErrors(){
		displayErrors(false);
	}

	protected final void displayErrors(boolean withStackTrace){
		SEDException[] array = new SEDException[exceptions.size()];
		for(int i=0; i<exceptions.size(); i++)
			array[i] = (SEDException)exceptions.get(i);
		if (fraErrorsReport == null)
			fraErrorsReport =  new ErrorReport((notifier != null && notifier instanceof JFrame)?(JFrame)notifier:null, this, withStackTrace);
		fraErrorsReport.setVisible(true);
	}

	public final void writeLog(File parentDir){
		if (exceptions.isEmpty())
			return;

		try {
			SimpleDateFormat sdf = new SimpleDateFormat("ddMMyy-HHmm");
			String logFile = "SEDPluginErrors_"+sdf.format(new Date())+".log";
			BufferedWriter writer = new BufferedWriter(new FileWriter(new File(parentDir, logFile)));

			boolean refreshError = false;
			for(int i=0; i<exceptions.size(); i++){
				SEDException ex = (SEDException)exceptions.get(i);
				if (refreshError = (ex instanceof RefreshException))
					ex = ((RefreshException)ex).getError();
				writer.write("##### "+(refreshError?"[REFRESH ERROR]":"")+" "+ex.getTitle()+" #####\n");
				writer.write("Origin: "+ex.getOrigin()+"\n");
				writer.write("Description: "+ex.getMessage()+"\n");
				writer.write("Stack trace: \n");
				writer.write(getStackTraceString(ex));
				writer.write("\n\n");
			}

			writer.close();
		}catch (java.io.IOException e) {
			addException(new SEDException(e, "Serialization of all exceptions"));
		}
	}

	public static final String getStackTraceString(Throwable ex){
		StringBuffer stackTrace = new StringBuffer();
		StackTraceElement[] elements = ex.getStackTrace();
		stackTrace.append("### ").append(ex.getClass().getName()).append(" ###\n");
		for(int i=0; i<elements.length; i++)
			stackTrace.append(elements[i].toString()).append("\n");

		if (ex.getCause() != null){
			elements = ex.getCause().getStackTrace();
			stackTrace.append("### CAUSED BY ").append(ex.getCause().getClass().getName()).append(" ###\n");
			for(int i=0; i<elements.length; i++)
				stackTrace.append(elements[i].toString()).append("\n");
		}

		return stackTrace.toString();
	}
}
