package sedplugin.interfaceGraphique;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.net.URL;

import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;



import sedplugin.errors.SEDException;
import sedplugin.plugin.SEDPlugin;
import sedplugin.sed.SED;
import sedplugin.sed.SEDPoint;
import sedplugin.sed.source.SEDSource;
import sedplugin.sed.source.aladin.Circle;
import sedplugin.sed.source.aladin.Polygon;
import sedplugin.sed.source.operation.Operation;
import sedplugin.sed.source.operation.OperationToken;


/**
 * A frame to make operations (complex or not) between spectrums and/or numeric values.<BR />
 * If the result is a numeric value it will be shown in a simple message dialog.
 * But if it's a spectrum the result will be drawn in a new frame (because of the difficulty to predict resulting ordinate unit.
 *
 * @see VisuGraph
 *
 * @author Gregory Mantelet
 * @version 04/2012
 */
public class CalculatorDialog extends JFrame {

	private static final long serialVersionUID = 1L;

	protected int nbComputedSpectrums;

	// NAME:
	protected JTextField txtName;

	// EXPRESSION:
	protected JTextArea txtExpr;

	// OPERANDS:
	protected JList lstSEDs;

	// OPERATION:
	protected JPanel panOperations;
	protected JButton cmdReset;
	protected JButton cmdSum;
	protected JButton cmdSub;
	protected JButton cmdMult;
	protected JButton cmdDiv;
	protected JButton cmdPow;
	protected JButton cmdBrackets;

	// MAIN COMMANDS:
	protected JButton cmdEvaluate;
	protected JLabel lblOK;
	protected JButton cmdCancel;

	protected final SEDFrame parent;
	protected final SEDPlugin myPlugin;

	/** Icon for a spectrum coming from a position selection. */
	private static ImageIcon icoPos = null;

	/** Icon for a spectrum coming from a circle selection. */
	private static ImageIcon icoCircle = null;

	/** Icon for a spectrum coming from a polygon selection. */
	private static ImageIcon icoPolygon = null;

	/** Icon for a spectrum coming from a spectrums operation. */
	private static ImageIcon icoOperation = null;

	/** Icon for a spectrum coming from a catalog. */
	private static ImageIcon icoCatalog = null;


	/* ************ */
	/* CONSTRUCTORS */
	/* ************ */
	public CalculatorDialog(SEDFrame owner, SEDPlugin plug){
		// Manage parameters:
		parent = owner;
		myPlugin = plug;
		nbComputedSpectrums = 0;

		// Build GI
		build();

		// Display dialog:
		setLocationRelativeTo(owner);
		setVisible(false);
	}

	/* **************** */
	/* BUILDING METHODS */
	/* **************** */
	private void build(){
		buildSEDIcon();

		// NAME FIELD:
		JLabel lblName = new JLabel("Name: ");
		lblName.setFont(lblName.getFont().deriveFont(Font.BOLD));
		txtName = new JTextField("Computed spectrum (0)");
		txtName.setFont(txtName.getFont().deriveFont(Font.PLAIN));
		JPanel panName = new JPanel(new BorderLayout());
		panName.add(lblName, BorderLayout.WEST);
		panName.add(txtName, BorderLayout.CENTER);

		// EXPRESSION AREA:
		txtExpr = new JTextArea(6, 30);
		txtExpr.addKeyListener(new ExprKeyListener());
		txtExpr.setFont(txtExpr.getFont().deriveFont(Font.PLAIN));
		txtExpr.setTabSize(4);
		JPanel panExpr = new JPanel(new BorderLayout());
		panExpr.setBorder(BorderFactory.createTitledBorder("Expression"));
		panExpr.add(new JScrollPane(txtExpr), BorderLayout.CENTER);

		// OPERATIONS TOOLBAR:
		// Clear button:
		cmdReset = new JButton("Clear");
		cmdReset.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				txtExpr.setText("");
				txtExpr.requestFocusInWindow();
			}
		});
		cmdReset.setForeground(Color.RED);
		cmdReset.setFont(cmdReset.getFont().deriveFont(Font.ITALIC|Font.BOLD, 16));
		// SUM button:
		cmdSum = new JButton("+");
		cmdSum.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				txtExpr.replaceSelection(" + ");
				txtExpr.requestFocusInWindow();
			}
		});
		cmdSum.setFont(cmdSum.getFont().deriveFont(Font.BOLD, 20));
		// SUBSTRACT button:
		cmdSub = new JButton("-");
		cmdSub.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				txtExpr.replaceSelection(" - ");
				txtExpr.requestFocusInWindow();
			}
		});
		cmdSub.setFont(cmdSub.getFont().deriveFont(Font.BOLD, 20));
		// MULTIPLICATE button:
		cmdMult = new JButton("×");
		cmdMult.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				txtExpr.replaceSelection(" * ");
				txtExpr.requestFocusInWindow();
			}
		});
		cmdMult.setFont(cmdMult.getFont().deriveFont(Font.BOLD, 20));
		// DIVIDE button:
		cmdDiv = new JButton("÷");
		cmdDiv.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				txtExpr.replaceSelection(" / ");
				txtExpr.requestFocusInWindow();
			}
		});
		cmdDiv.setFont(cmdDiv.getFont().deriveFont(Font.BOLD, 20));
		// POWER button:
		cmdPow = new JButton("^");
		cmdPow.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				txtExpr.replaceSelection(" ^ ");
				txtExpr.requestFocusInWindow();
			}
		});
		cmdPow.setFont(cmdPow.getFont().deriveFont(Font.BOLD, 20));
		// BRACKETS button:
		cmdBrackets = new JButton("()");
		cmdBrackets.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				txtExpr.replaceSelection("()");
				txtExpr.setCaretPosition(txtExpr.getCaretPosition()-1);
				txtExpr.requestFocusInWindow();
			}
		});
		cmdBrackets.setFont(cmdBrackets.getFont().deriveFont(Font.BOLD, 20));
		// Tool bar:
		panOperations = new JPanel(new GridBagLayout());
		// general constraints:
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0; c.gridy = 0; c.gridwidth = 2; c.gridheight = 1;
		panOperations.add(cmdReset, c);
		c.gridx = 0; c.gridy = 1; c.gridwidth = 1; c.gridheight = 1;
		panOperations.add(cmdSum, c);
		c.gridx = 1; c.gridy = 1;
		panOperations.add(cmdSub, c);
		c.gridx = 0; c.gridy = 2;
		panOperations.add(cmdMult, c);
		c.gridx = 1; c.gridy = 2;
		panOperations.add(cmdDiv, c);
		c.gridx = 0; c.gridy = 3;
		panOperations.add(cmdPow, c);
		c.gridx = 1; c.gridy = 3;
		panOperations.add(cmdBrackets, c);

		// OPERAND PANEL:
		// SED list:
		Vector<SED> seds = new Vector<SED>();
		Iterator<SED> it = myPlugin.getSEDs();
		while(it.hasNext())
			seds.add(it.next());
		lstSEDs = new JList(seds);
		lstSEDs.setOpaque(true);
		lstSEDs.setBackground(Color.LIGHT_GRAY);
		lstSEDs.setBorder(BorderFactory.createEmptyBorder());
		lstSEDs.setCellRenderer(new MySpectrumRenderer());
		lstSEDs.setVisibleRowCount(4);
		lstSEDs.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		lstSEDs.addListSelectionListener(new ListSelectionListener(){
			public void valueChanged(ListSelectionEvent e) {
				if (!e.getValueIsAdjusting() && lstSEDs.getSelectedIndex() > -1){
					txtExpr.replaceSelection("$"+(lstSEDs.getSelectedIndex()+1));
					txtExpr.requestFocusInWindow();
					lstSEDs.clearSelection();
				}
			}
		});
		lstSEDs.setFont(lstSEDs.getFont().deriveFont(Font.PLAIN));

		// MAIN COMMANDS:
		// Evaluate command:
		cmdEvaluate = new JButton("Evaluate [F5]");
		cmdEvaluate.addActionListener(new EvaluateActionListener());
		// Status field:
		lblOK = new JLabel("Syntax ?");
		lblOK.setOpaque(true);
		// Cancel command:
		cmdCancel = new JButton("Close [Esc]");
		cmdCancel.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				hideCalculator();
			}
		});
		// PanCommands:
		JPanel panCommands = new JPanel();
		panCommands.add(cmdEvaluate);
		panCommands.add(lblOK);
		panCommands.add(cmdCancel);

		// CONTENT PANEL:
		JPanel panContent = new JPanel(new BorderLayout());
		panContent.add(panName, BorderLayout.NORTH);
		panContent.add(panExpr, BorderLayout.SOUTH);
		panContent.add(new JScrollPane(lstSEDs), BorderLayout.CENTER);
		panContent.add(panOperations, BorderLayout.EAST);

		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(panContent, BorderLayout.CENTER);
		getContentPane().add(panCommands, BorderLayout.SOUTH);

		setTitle("SED Calculator");
		URL iconTitle = getClass().getResource(SEDPlugin.ICONS_DIR+"calculator.gif");
		if (iconTitle != null)
			setIconImage(new ImageIcon(iconTitle).getImage());
		pack();
		setResizable(false);
	}

	private final void buildSEDIcon(){
		// Load all required icons:
		if (icoPos == null){
			URL urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"position.gif");
			if (urlIcon != null)
				icoPos = new ImageIcon(urlIcon);
		}
		if (icoCircle == null){
			URL urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"circle.gif");
			if (urlIcon != null)
				icoCircle = new ImageIcon(urlIcon);
		}
		if (icoPolygon == null){
			URL urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"polygon.gif");
			if (urlIcon != null)
				icoPolygon = new ImageIcon(urlIcon);
		}
		if (icoOperation == null){
			URL urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"calculator.gif");
			if (urlIcon != null)
				icoOperation = new ImageIcon(urlIcon);
		}
		if (icoCatalog == null){
			URL urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"source3.png");
			if (urlIcon != null)
				icoCatalog = new ImageIcon(urlIcon);
		}
	}

	/* ***************** */
	/* WINDOW MANAGEMENT */
	/* ***************** */
	protected void hideCalculator(){
		setVisible(false);
	}

	/* *************** */
	/* COMPUTE METHODS */
	/* *************** */
	protected void evaluate(){
		if (txtExpr.getText().trim().length() <= 0) return;
		try{
			Operation op = new Operation(txtExpr.getText(), myPlugin);
			// Compute the expression and get its result:
			OperationToken result = op.getResult();
			// Refresh the status of the evaluation:
			lblOK.setBackground(Color.GREEN);
			lblOK.setText("SUCCESS");
			String resultString = "<html><p align=\"center\"><b>EVALUATED</b></p><p align=\"center\"><code>"+op+"</code> <b>=</b> ";
			switch(result.type){
			case CONSTANT:
				resultString += result+"</p>";
				break;
			case SED:
				SED sp = op.getSED();
				// set the spectrum name:
				if (txtName.getText().trim().length() <= 0)
					sp.setName(op+"");
				else
					sp.setName(txtName.getText());
				resultString += "</p><table border=\"1px\" align=\"center\"><tr><th>"+sp.getXAxis().getUnit().symbol+"</th><th>"+sp.getYAxis().getUnit().symbol+"</th></tr>";
				for(SEDPoint spt : sp)
					resultString += "<tr><td>"+spt.getX()+"</td><td>"+spt.getY()+"</td></tr>";
				resultString += "</table>";
				break;
			}
			lblOK.setToolTipText(resultString+"</html>");
			// Print/Add the result:
			switch(result.type){
			case CONSTANT:
				JOptionPane.showMessageDialog(CalculatorDialog.this, "<html><code>"+op+"</code> = <b>"+result+"</b></html>", "RESULT", JOptionPane.INFORMATION_MESSAGE);
				break;
			case SED:
				if (op.getSED() != null && !op.getSED().isEmpty()){
					myPlugin.addSEDSource(op);
					myPlugin.showSED(op.getSED());
					nbComputedSpectrums++;
				}
				txtName.setText("Computed spectrum ("+nbComputedSpectrums+")");
				break;
			}
		}catch(SEDException se){
			lblOK.setBackground(Color.RED);
			lblOK.setText("PARSING");
			lblOK.setToolTipText("<html>"+se.toHTMLString()+"</html>");
			JOptionPane.showMessageDialog(CalculatorDialog.this, se.getMessage(), se.getTitle(), JOptionPane.ERROR_MESSAGE);
		}
	}

	/* ********* */
	/* LISTENERS */
	/* ********* */
	private final class EvaluateActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			evaluate();
		}
	}

	private final class ExprKeyListener implements KeyListener {
		public void keyTyped(KeyEvent e) {
			if (e.getKeyChar() == '⁰'){
				txtExpr.replaceSelection(" ^0 ");
				e.consume();
			}else if (e.getKeyChar() == '¹'){
				txtExpr.replaceSelection(" ^1 ");
				e.consume();
			}else if (e.getKeyChar() == '²'){
				txtExpr.replaceSelection(" ^2 ");
				e.consume();
			}else if (e.getKeyChar() == '³'){
				txtExpr.replaceSelection(" ^3 ");
				e.consume();
			}else if (e.getKeyChar() == '⁴'){
				txtExpr.replaceSelection(" ^4 ");
				e.consume();
			}else if (e.getKeyChar() == '⁵'){
				txtExpr.replaceSelection(" ^5 ");
				e.consume();
			}else if (e.getKeyChar() == '⁶'){
				txtExpr.replaceSelection(" ^6 ");
				e.consume();
			}else if (e.getKeyChar() == '⁷'){
				txtExpr.replaceSelection(" ^7 ");
				e.consume();
			}else if (e.getKeyChar() == '⁸'){
				txtExpr.replaceSelection(" ^8 ");
				e.consume();
			}else if (e.getKeyChar() == '⁹'){
				txtExpr.replaceSelection(" ^9 ");
				e.consume();
			}
		}
		public void keyPressed(KeyEvent e) {
			if (e.getKeyCode() == KeyEvent.VK_F5)
				evaluate();
			else if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
				hideCalculator();
		}
		public void keyReleased(KeyEvent e) {

		}
	}

	public class MySpectrumRenderer extends JButton implements ListCellRenderer {

		private static final long serialVersionUID = 1L;

		public MySpectrumRenderer(){
			setMinimumSize(new Dimension(100,20));
			setMaximumSize(new Dimension(Integer.MAX_VALUE, 20));
		}

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			setSelected(isSelected);

			SED sed = (SED)value;
			Color color = myPlugin.getColor(sed);

			setText("<html><b>"+value.toString()+"</b><i> ($"+(index+1)+")</i></html>");

			// set an icon according to the spectrum origin:
			if (sed.getSource() != null){
				SEDSource src = sed.getSource();
				String srcType = src.getType();
				if (srcType.equals(Circle.CIRCLE_TYPE)){
					setIcon(icoCircle);
					color = ((Circle)src).getColor();
				}else if (srcType.equals(Polygon.POLYGON_TYPE)){
					setIcon(icoPolygon);
					color = ((Polygon)src).getColor();
				}else if (srcType.equals(Operation.OPERATION_TYPE))
					setIcon(icoOperation);
				else
					setIcon(null);
			}

			// set the text color:
			setForeground(color);

			return this;
		}
	}
}
