package sedplugin.interfaceGraphique;

import java.io.File;

import javax.swing.filechooser.FileFilter;

public class ExportFileFilter extends FileFilter {
	
	public final String filterName;
	public final String[] types;
	
	public ExportFileFilter(String extension){
		this(extension, null);
	}
	
	public ExportFileFilter(String[] extensions){
		this(extensions, null);
	}
	
	public ExportFileFilter(String extension, String name){
		this(new String[]{extension}, name);
	}
	
	public ExportFileFilter(String[] extensions, String name){
		types = extensions;
		filterName = name;
	}
	
	public final String getFilterName(){
		return filterName;
	}
	
	public final String getExtension(){
		return types[0];
	}
	
	public final String[] getExtensions(){
		return types;
	}
	
	public final boolean accept(File f) {
		if (f.isDirectory())
			return true;
		
		String ext = getExtension(f);
		if (ext == null)
			return false;
		
		boolean accepted = false;
		for(int i=0; i<types.length && !accepted; i++)
			accepted = ext.equalsIgnoreCase(types[i]);
			
		return accepted;
	}

	public final String getDescription() {
		String desc = "";
		
		if (filterName == null){
			if (types.length > 0)
				desc = "."+types[0].toUpperCase();
			for(int i=1; i<types.length; i++)
				desc += "; ."+types[i].toUpperCase();
		}else
			desc = filterName;
		
		return desc;
	}
	
	public static final String getExtension(File f){
		String ext = f.getName();
		int index = ext.lastIndexOf('.')+1;
		if (index <= 0 || index >= ext.length())
			return null;
		else
			return ext.substring(index).trim().toLowerCase();
	}

}
