package sedplugin.interfaceGraphique;

import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.Timer;

import sedplugin.errors.RefreshException;
import sedplugin.errors.SEDException;
import sedplugin.plugin.SEDPlugin;


public class NotificationLabel extends JLabel {

	private static final long serialVersionUID = 1L;

	/** Icons to use to specify the gravity of the given exception. */
	protected static ImageIcon icoError = null;
	protected static ImageIcon icoRefreshError = null;

	protected Timer timDissolve;

	int horizontalLocation = JLabel.TOP;
	int verticalLocation = JLabel.LEFT;

	int xShift = 0;
	int yShift = 0;

	public NotificationLabel() {
		super();
		build();
	}

	public NotificationLabel(String text) {
		super(text);
		build();
	}

	public NotificationLabel(Icon icon) {
		super(icon);
		build();
	}

	public NotificationLabel(String text, int horizontalAlignement) {
		super(text, horizontalAlignement);
		build();
	}

	public NotificationLabel(Icon icon, int horizontalAlignement) {
		super(icon, horizontalAlignement);
		build();
	}

	public NotificationLabel(String text, Icon icon, int horizontalAlignement) {
		super(text, icon, horizontalAlignement);
		build();
	}

	protected void build(){
		setOpaque(true);

		timDissolve = new Timer(100, new MsgHidder());
		timDissolve.setInitialDelay(5000);

		URL iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR+"error.gif");
		icoError = (iconUrl == null) ? null : new ImageIcon(iconUrl);
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR+"synchronizeError.png");
		icoRefreshError = (iconUrl == null) ? null : new ImageIcon(iconUrl);
	}

	public void setNotifyAlignement(int xAlignementInContainer, int yAlignementInContainer){
		if (xAlignementInContainer == JLabel.LEFT || xAlignementInContainer == JLabel.RIGHT || xAlignementInContainer == JLabel.CENTER)
			horizontalLocation = xAlignementInContainer;

		if (yAlignementInContainer == JLabel.TOP || yAlignementInContainer == JLabel.BOTTOM || yAlignementInContainer == JLabel.CENTER)
			verticalLocation = yAlignementInContainer;
	}

	public void setNotifyLocation(int horizontalShift, int verticalShift){
		xShift = horizontalShift;
		yShift = verticalShift;
	}

	public void setNotifyLocation(int horizontalShift, int verticalShift, int xAlignement, int yAlignement){
		setNotifyAlignement(xAlignement, yAlignement);
		setNotifyLocation(horizontalShift, verticalShift);
	}

	public synchronized void printException(Exception ex){
		timDissolve.stop();

		if (ex instanceof SEDException){
			SEDException ex2 = (SEDException)ex;
			boolean refreshError = (ex2 instanceof RefreshException);
			if (refreshError)
				ex2 = ((RefreshException)ex2).getError();
			setIcon(refreshError ? icoRefreshError : icoError);
			setBorder(BorderFactory.createLineBorder(refreshError ? Color.ORANGE : Color.RED));
			setBackground(refreshError ? new Color(246,235,155, 255) : new Color(241,184,168, 255));
			setFont(getFont().deriveFont(Font.PLAIN));
			// Show the title of the error:
			setText(ex2.getTitle());
			setToolTipText(ex.getMessage());
			setForeground(new Color(0, 0, 0, 255));
		}else{
			// Show the gravity icon:
			setIcon(icoError);
			setBorder(BorderFactory.createLineBorder(Color.RED));
			setBackground(new Color(241,184,168, 255));
			setFont(getFont().deriveFont(Font.BOLD));
			// Show the title of the error:
			setText(ex.getMessage());
			setToolTipText(ex.getMessage());
		}

		updateBounds();

		// Start the timer:
		startShading();
	}

	@Override
	public void setIcon(Icon ico) {
		super.setIcon(ico);
	}

	@Override
	public void setText(String txt) {
		super.setText(txt);
	}

	protected void updateBounds(){
		int x = xShift, y = yShift, width, height;
		FontMetrics fm = getFontMetrics(getFont());

		width = ((getIcon()==null)?0:getIcon().getIconWidth())+fm.stringWidth(getText())+20;
		height = Math.max((getIcon()==null)?0:getIcon().getIconHeight(), fm.getHeight());

		if (getParent() != null){
			if (width > getParent().getWidth()/2) width = getParent().getWidth()/2;

			switch(horizontalLocation){
			case JLabel.RIGHT:
				x = ((getParent() == null)?0:getParent().getWidth())-xShift-width; break;
			case JLabel.CENTER:
				x = ((((getParent() == null)?0:getParent().getWidth())-width-xShift)/2); break;
			}

			switch(verticalLocation){
			case JLabel.BOTTOM:
				y = ((getParent() == null)?0:getParent().getHeight())-yShift-height; break;
			case JLabel.CENTER:
				y = (((getParent() == null)?0:getParent().getHeight())-height-yShift)/2; break;
			}
		}

		setBounds(x, y, width, height);
	}

	public void startShading(){
		timDissolve.stop();

		if (getParent() != null){
			setVisible(true);
			getParent().repaint();
			timDissolve.start();
		}
	}

	private final class MsgHidder implements ActionListener {
		protected int alpha = 255;
		protected int step;

		public MsgHidder(){
			step = alpha / 10;
		}

		public void actionPerformed(ActionEvent evt) {
			JLabel lblMsg = NotificationLabel.this;
			Container container = lblMsg.getParent();

			alpha -= step;

			if (alpha < 10){
				alpha = 255;
				timDissolve.stop();
				lblMsg.setVisible(false);
				container.repaint();
			}else{
				Color col = lblMsg.getForeground();
				lblMsg.setForeground(new Color(col.getRed(), col.getGreen(), col.getBlue(), alpha));
				col = lblMsg.getBackground();
				lblMsg.setBackground(new Color(col.getRed(), col.getGreen(), col.getBlue(), alpha));
				container.repaint();
			}
		}
	}

}
