package sedplugin.interfaceGraphique;



import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;

import sedplugin.graphe.Point;
import sedplugin.interfaceGraphique.SEDFrame.ColorRenderer;

/**
 * A ListCellRenderer which renders a {@link Point} object as a JPanel
 * in which the graphical representation of the given point is drawn.
 * 
 * @see Point
 * 
 * @author Gregory Mantelet
 * @version 01/2010
 */
public class PointRenderer extends JPanel implements ListCellRenderer {

	private static final long serialVersionUID = 1L;
	
	/** Width and height of the JPanel component. */
	public static final int WIDTH = 20, HEIGHT = 20;
	
	/** The point to render. */
	protected Point pt = null;
	
	/** The {@link ColorRenderer} which manages the serie color. */
	protected ColorRenderer serieColorRenderer;

/* ************ */
/* CONSTRUCTORS */
/* ************ */
	/**
	 * Initialize this PointRenderer.<BR />
	 * The points color will be BLACK.
	 */
	public PointRenderer(){
		this(null);
	}
	
	/**
	 * Initialize this PointRenderer.
	 * @param serieColor	The {@link ColorRenderer} which manages the serie color.
	 */
	public PointRenderer(ColorRenderer serieColor) {
		serieColorRenderer = serieColor;
		setOpaque(true);
		setBackground(Color.WHITE);
		setPreferredSize(new Dimension(WIDTH, HEIGHT));
		setMinimumSize(new Dimension(WIDTH, HEIGHT));
		setMaximumSize(new Dimension(WIDTH, HEIGHT));
		setBorder(BorderFactory.createRaisedBevelBorder());
		setSize(new Dimension(WIDTH, HEIGHT));
	}
	
/* ***************** */
/* GETTERS & SETTERS */
/* ***************** */
	/**
	 * Gets the position of any point rendered by this ListCellRenderer.
	 * @return	The position of any point in this renderer.
	 */
	public static final int[] getPointPosition(){
		return new int[]{WIDTH/2, HEIGHT/2};
	}

/* ***************** */
/* INHERITED METHODS */
/* ***************** */
	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
		pt = (Point)value;
		setBackground((isSelected)?Color.LIGHT_GRAY:Color.WHITE);
		
		repaint();
		return this;
	}

/* ************ */
/* DRAW METHODS */
/* ************ */
	public void paint(Graphics g) {
		super.paint(g);
		if (pt != null){
			Graphics2D g2 = (Graphics2D)g;
			RenderingHints rh = g2.getRenderingHints();
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			Color oldColor = g.getColor();
			g.setColor((serieColorRenderer!=null)?serieColorRenderer.getColor():Color.BLACK);
			pt.draw(g);
			g.setColor(oldColor);
			g2.setRenderingHints(rh);
		}
		g.dispose();
	}

}
