package sedplugin.interfaceGraphique;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.WindowConstants;

import sedplugin.plugin.SEDPlugin;
import sedplugin.plugin.UserPreferences;

/**
 * A dialog to let the user modify some plugin settings.
 *
 * @see UserPreferences
 *
 * @author Gregory Mantelet
 * @version 01/2010
 */
public class PreferencesDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	protected JTabbedPane tabs;

	protected JRadioButton optInteractionsNever;
	protected JRadioButton optInteractionsAfterEachOne;

	protected JRadioButton optImageIgnore;
	protected JRadioButton optImageRemove;
	protected JCheckBox chkDisplayImageErrors;

	protected JComboBox cboGravities;
	protected JCheckBox chkSaveErrorsLog;

	protected JRadioButton optSyncOnAnyAladinModification;
	protected JRadioButton optSyncOnActivation;
	protected JRadioButton optSyncOnUser;

	protected JRadioButton optSyncPbm_Ask;
	protected JRadioButton optSyncPbm_DefaultAction;
	protected JRadioButton optSyncPbm_Nothing;

	protected JCheckBox chkClickOnImage;
	protected JCheckBox chkMoveToOnSpectrumClick;
	protected JCheckBox chkPrintPointValuesOnOver;
	protected JCheckBox chkAskConfirmationBeforeSendToSamp;

	protected JButton cmdOK;

	/* ************ */
	/* CONSTRUCTORS */
	/* ************ */
	public PreferencesDialog(JFrame owner){
		super(owner);

		getContentPane().setLayout(new BorderLayout());

		tabs = new JTabbedPane();

		// Add the synchronization options tab:
		URL iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR+"synchronize.png");
		tabs.addTab("Interaction", (iconUrl != null)?new ImageIcon(iconUrl):null, getInteractionTab(), "Options about interaction with Aladin");
		tabs.setMnemonicAt(0, KeyEvent.VK_I);

		// Add the synchronization options tab:
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR+"refresh.gif");
		tabs.addTab("Synchronization", (iconUrl != null)?new ImageIcon(iconUrl):null, getSynchronizationTab(), "Options about synchronization with the Aladin stack");
		tabs.setMnemonicAt(1, KeyEvent.VK_S);

		// Add the general options tab:
		/*iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR+"littleWarning.gif");
		tabs.addTab("Errors", (iconUrl != null)?new ImageIcon(iconUrl):null, getErrorsTab(), "Errors management");
		tabs.setMnemonicAt(2, KeyEvent.VK_E);*/

		// Add the both tabs:
		getContentPane().add(tabs, BorderLayout.CENTER);

		// Add commands panel:
		JPanel panCommands = new JPanel();
		iconUrl = getClass().getResource(SEDPlugin.ICONS_DIR+"ok.gif");
		if (iconUrl == null)
			cmdOK = new JButton("OK");
		else
			cmdOK = new JButton("OK", new ImageIcon(iconUrl));
		cmdOK.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		panCommands.add(cmdOK);
		getContentPane().add(panCommands, BorderLayout.SOUTH);

		addWindowListener(new WindowAdapter() {
			@Override
			public void windowActivated(WindowEvent e) {
				super.windowActivated(e);
				tabs.repaint();
			}
		});

		// General preferences:
		setTitle("Preferences");
		setSize(450,330);
		setResizable(false);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		setLocationRelativeTo(owner);
		setModal(true);
		setVisible(false);
	}



	/* **************** */
	/* BUILDING METHODS */
	/* **************** */
	/*
	 * Builds the general options tab.
	 * @return	The general options tab.
	 *
	private JPanel getErrorsTab(){
		JPanel panGeneralOptions = new JPanel(new GridLayout(2,1));

	// General errors management:
		cboGravities = new JComboBox(new String[]{">= WARNING", ">= ERROR", "CRITICAL ERROR"});
		cboGravities.setEditable(false);
		cboGravities.setSelectedIndex(UserPreferences.getCurrentInstance().displayErrorsIfGravityIs());
		cboGravities.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if (cboGravities.getSelectedIndex() > -1)
					UserPreferences.getCurrentInstance().displayErrorsIfGravityIs(cboGravities.getSelectedIndex());
			}
		});
		JPanel panGravity = new JPanel(new BorderLayout());
		panGravity.add(new JLabel("Display errors whose the gravity is "), BorderLayout.WEST);
		panGravity.add(cboGravities, BorderLayout.CENTER);

		chkSaveErrorsLog = new JCheckBox("Save a log of errors before closing SEDPlugin");
		if (UserPreferences.getCurrentInstance().saveErrorsLogBeforeClosing()){
			chkSaveErrorsLog.setSelected(true);
			chkSaveErrorsLog.setFont(chkSaveErrorsLog.getFont().deriveFont(Font.BOLD));
		}else
			chkSaveErrorsLog.setFont(chkSaveErrorsLog.getFont().deriveFont(Font.PLAIN));
		chkSaveErrorsLog.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().saveErrorsLogBeforeClosing(chkSaveErrorsLog.isSelected());
				if (chkSaveErrorsLog.isSelected())
					chkSaveErrorsLog.setFont(chkSaveErrorsLog.getFont().deriveFont(Font.BOLD));
				else
					chkSaveErrorsLog.setFont(chkSaveErrorsLog.getFont().deriveFont(Font.PLAIN));
			}
		});

		JPanel panErrorsManagement = new JPanel(new GridLayout(2,1));
		panErrorsManagement.setBorder(BorderFactory.createTitledBorder("General errors management"));
		panErrorsManagement.add(panGravity);
		panErrorsManagement.add(chkSaveErrorsLog);

	// Image loading error:
		optImageIgnore = new JRadioButton("ignore image");
		if (UserPreferences.getCurrentInstance().onImageLoadingError() == UserPreferences.IGNORE_ALL){
			optImageIgnore.setSelected(true);
			optImageIgnore.setFont(optImageIgnore.getFont().deriveFont(Font.BOLD));
		}else
			optImageIgnore.setFont(optImageIgnore.getFont().deriveFont(Font.PLAIN));
		optImageIgnore.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().onImageLoadingError(UserPreferences.IGNORE_ALL);
				if (optImageIgnore.isSelected()){
					optImageIgnore.setFont(optImageIgnore.getFont().deriveFont(Font.BOLD));
					optImageRemove.setFont(optImageRemove.getFont().deriveFont(Font.PLAIN));
				}
			}
		});

		optImageRemove = new JRadioButton("remove image from the Aladin stack");
		if (UserPreferences.getCurrentInstance().onImageLoadingError() == UserPreferences.DELETE_IMAGE){
			optImageRemove.setSelected(true);
			optImageRemove.setFont(optImageRemove.getFont().deriveFont(Font.BOLD));
		}else
			optImageRemove.setFont(optImageRemove.getFont().deriveFont(Font.PLAIN));
		optImageRemove.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().onImageLoadingError(UserPreferences.DELETE_IMAGE);
				if (optImageRemove.isSelected()){
					optImageRemove.setFont(optImageRemove.getFont().deriveFont(Font.BOLD));
					optImageIgnore.setFont(optImageIgnore.getFont().deriveFont(Font.PLAIN));
				}
			}
		});

		ButtonGroup imageLoadingGroup = new ButtonGroup();
		imageLoadingGroup.add(optImageIgnore);
		imageLoadingGroup.add(optImageRemove);

		chkDisplayImageErrors = new JCheckBox("Display the error");
		if (UserPreferences.getCurrentInstance().displayImageLoadingErrors()){
			chkDisplayImageErrors.setSelected(true);
			chkDisplayImageErrors.setFont(chkDisplayImageErrors.getFont().deriveFont(Font.BOLD));
		}else
			chkDisplayImageErrors.setFont(chkDisplayImageErrors.getFont().deriveFont(Font.PLAIN));
		chkDisplayImageErrors.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().displayImageLoadingError(chkDisplayImageErrors.isSelected());
				if (chkDisplayImageErrors.isSelected())
					chkDisplayImageErrors.setFont(chkDisplayImageErrors.getFont().deriveFont(Font.BOLD));
				else
					chkDisplayImageErrors.setFont(chkDisplayImageErrors.getFont().deriveFont(Font.PLAIN));
			}
		});

		JPanel panImageLoading = new JPanel(new GridLayout(3,1));
		panImageLoading.setBorder(BorderFactory.createTitledBorder("When there is an error with an image..."));
		panImageLoading.add(optImageIgnore);
		panImageLoading.add(optImageRemove);
		panImageLoading.add(chkDisplayImageErrors);

		panGeneralOptions.add(panErrorsManagement);
		panGeneralOptions.add(panImageLoading);

		return panGeneralOptions;
	}*/

	private JPanel getInteractionTab(){
		JPanel panInteractionOptions = new JPanel(new GridLayout(2,1));

		//	Interactions deactivation:
		optInteractionsNever = new JRadioButton("Never");
		if (UserPreferences.getCurrentInstance().whenDeactivatingInteractions() == UserPreferences.NEVER){
			optInteractionsNever.setSelected(true);
			optInteractionsNever.setFont(optInteractionsNever.getFont().deriveFont(Font.BOLD));
		}else
			optInteractionsNever.setFont(optInteractionsNever.getFont().deriveFont(Font.PLAIN));
		optInteractionsNever.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().deactivateInteractionsOn(UserPreferences.NEVER);
				if (optInteractionsNever.isSelected()){
					optInteractionsNever.setFont(optInteractionsNever.getFont().deriveFont(Font.BOLD));
					optInteractionsAfterEachOne.setFont(optInteractionsAfterEachOne.getFont().deriveFont(Font.PLAIN));
				}
			}
		});

		optInteractionsAfterEachOne = new JRadioButton("After each spectrum generation");
		if (UserPreferences.getCurrentInstance().whenDeactivatingInteractions() == UserPreferences.AFTER_EACH_ONE){
			optInteractionsAfterEachOne.setSelected(true);
			optInteractionsAfterEachOne.setFont(optInteractionsAfterEachOne.getFont().deriveFont(Font.BOLD));
		}else
			optInteractionsAfterEachOne.setFont(optInteractionsAfterEachOne.getFont().deriveFont(Font.PLAIN));
		optInteractionsAfterEachOne.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().deactivateInteractionsOn(UserPreferences.AFTER_EACH_ONE);
				if (optInteractionsAfterEachOne.isSelected()){
					optInteractionsAfterEachOne.setFont(optInteractionsAfterEachOne.getFont().deriveFont(Font.BOLD));
					optInteractionsNever.setFont(optInteractionsNever.getFont().deriveFont(Font.PLAIN));
				}
			}
		});

		ButtonGroup interactionsGroup = new ButtonGroup();
		interactionsGroup.add(optInteractionsNever);
		interactionsGroup.add(optInteractionsAfterEachOne);

		JPanel panInteractions = new JPanel(new GridLayout(2,1));
		panInteractions.setBorder(BorderFactory.createTitledBorder("When will interactions with Aladin have to be deactivated ?"));
		optInteractionsNever.setAlignmentX(Component.LEFT_ALIGNMENT);
		panInteractions.add(optInteractionsNever);
		optInteractionsAfterEachOne.setAlignmentX(Component.LEFT_ALIGNMENT);
		panInteractions.add(optInteractionsAfterEachOne);


		// Binary Options:
		/*chkClickOnImage = new JCheckBox("Create a SED at a click on an image");
		if (UserPreferences.getCurrentInstance().createSEDOnClick()){
			chkClickOnImage.setSelected(true);
			chkClickOnImage.setFont(chkClickOnImage.getFont().deriveFont(Font.BOLD));
		}else
			chkClickOnImage.setFont(chkClickOnImage.getFont().deriveFont(Font.PLAIN));
		chkClickOnImage.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().setCreateSEDOnClick(chkClickOnImage.isSelected());
				if (chkClickOnImage.isSelected())
					chkClickOnImage.setFont(chkClickOnImage.getFont().deriveFont(Font.BOLD));
				else
					chkClickOnImage.setFont(chkClickOnImage.getFont().deriveFont(Font.PLAIN));
			}
		});*/

		chkMoveToOnSpectrumClick = new JCheckBox("Move the Aladin reticule when clicking on a SED");
		if (UserPreferences.getCurrentInstance().moveToOnSpectrumClick()){
			chkMoveToOnSpectrumClick.setSelected(true);
			chkMoveToOnSpectrumClick.setFont(chkMoveToOnSpectrumClick.getFont().deriveFont(Font.BOLD));
		}else
			chkMoveToOnSpectrumClick.setFont(chkMoveToOnSpectrumClick.getFont().deriveFont(Font.PLAIN));
		chkMoveToOnSpectrumClick.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().setMoveToOnSpectrumClick(chkMoveToOnSpectrumClick.isSelected());
				if (chkMoveToOnSpectrumClick.isSelected())
					chkMoveToOnSpectrumClick.setFont(chkMoveToOnSpectrumClick.getFont().deriveFont(Font.BOLD));
				else
					chkMoveToOnSpectrumClick.setFont(chkMoveToOnSpectrumClick.getFont().deriveFont(Font.PLAIN));
			}
		});

		chkPrintPointValuesOnOver = new JCheckBox("Print spectral point value when the mouse is over");
		if (UserPreferences.getCurrentInstance().printValuesOnOver()){
			chkPrintPointValuesOnOver.setSelected(true);
			chkPrintPointValuesOnOver.setFont(chkPrintPointValuesOnOver.getFont().deriveFont(Font.BOLD));
		}else
			chkPrintPointValuesOnOver.setFont(chkPrintPointValuesOnOver.getFont().deriveFont(Font.PLAIN));
		chkPrintPointValuesOnOver.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().setPrintValuesOnOver(chkPrintPointValuesOnOver.isSelected());
				if (chkPrintPointValuesOnOver.isSelected())
					chkPrintPointValuesOnOver.setFont(chkPrintPointValuesOnOver.getFont().deriveFont(Font.BOLD));
				else
					chkPrintPointValuesOnOver.setFont(chkPrintPointValuesOnOver.getFont().deriveFont(Font.PLAIN));
			}
		});

		chkAskConfirmationBeforeSendToSamp = new JCheckBox("Ask confirmation before sending a SED");
		if (UserPreferences.getCurrentInstance().askConfirmationBeforeSendToSamp()){
			chkAskConfirmationBeforeSendToSamp.setSelected(true);
			chkAskConfirmationBeforeSendToSamp.setFont(chkAskConfirmationBeforeSendToSamp.getFont().deriveFont(Font.BOLD));
		}else
			chkAskConfirmationBeforeSendToSamp.setFont(chkAskConfirmationBeforeSendToSamp.getFont().deriveFont(Font.PLAIN));
		chkAskConfirmationBeforeSendToSamp.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().setAskConfirmationBeforeSendToSamp(chkAskConfirmationBeforeSendToSamp.isSelected());
				if (chkAskConfirmationBeforeSendToSamp.isSelected())
					chkAskConfirmationBeforeSendToSamp.setFont(chkAskConfirmationBeforeSendToSamp.getFont().deriveFont(Font.BOLD));
				else
					chkAskConfirmationBeforeSendToSamp.setFont(chkAskConfirmationBeforeSendToSamp.getFont().deriveFont(Font.PLAIN));
			}
		});

		JPanel binaryOptions = new JPanel(new GridLayout(3,1));
		//binaryOptions.add(chkClickOnImage);
		binaryOptions.add(chkMoveToOnSpectrumClick);
		binaryOptions.add(chkPrintPointValuesOnOver);
		binaryOptions.add(chkAskConfirmationBeforeSendToSamp);

		panInteractionOptions.add(panInteractions);
		panInteractionOptions.add(binaryOptions);

		return panInteractionOptions;
	}


	/**
	 * Builds the synchronization options tab.
	 * @return	The synchronization options tab.
	 */
	private JPanel getSynchronizationTab(){
		JPanel panSynchronization = new JPanel(new GridLayout(1,1));

		// Refresh all on:
		optSyncOnAnyAladinModification = new JRadioButton("On any Aladin stack modification");
		if (UserPreferences.getCurrentInstance().whenAllRefreshing() == UserPreferences.ON_ANY_ALADIN_MODIFICATION){
			optSyncOnAnyAladinModification.setSelected(true);
			optSyncOnAnyAladinModification.setFont(optSyncOnAnyAladinModification.getFont().deriveFont(Font.BOLD));
		}else
			optSyncOnAnyAladinModification.setFont(optSyncOnAnyAladinModification.getFont().deriveFont(Font.PLAIN));
		optSyncOnAnyAladinModification.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().setRefreshAllOn(UserPreferences.ON_ANY_ALADIN_MODIFICATION);
				if (optSyncOnAnyAladinModification.isSelected()){
					optSyncOnAnyAladinModification.setFont(optSyncOnAnyAladinModification.getFont().deriveFont(Font.BOLD));
					optSyncOnActivation.setFont(optSyncOnActivation.getFont().deriveFont(Font.PLAIN));
					optSyncOnUser.setFont(optSyncOnUser.getFont().deriveFont(Font.PLAIN));
				}
			}
		});

		optSyncOnActivation = new JRadioButton("On plugin activation");
		if (UserPreferences.getCurrentInstance().whenAllRefreshing() == UserPreferences.ON_ACTIVATE){
			optSyncOnActivation.setSelected(true);
			optSyncOnActivation.setFont(optSyncOnActivation.getFont().deriveFont(Font.BOLD));
		}else
			optSyncOnActivation.setFont(optSyncOnActivation.getFont().deriveFont(Font.PLAIN));
		optSyncOnActivation.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().setRefreshAllOn(UserPreferences.ON_ACTIVATE);
				if (optSyncOnActivation.isSelected()){
					optSyncOnActivation.setFont(optSyncOnActivation.getFont().deriveFont(Font.BOLD));
					optSyncOnAnyAladinModification.setFont(optSyncOnAnyAladinModification.getFont().deriveFont(Font.PLAIN));
					optSyncOnUser.setFont(optSyncOnUser.getFont().deriveFont(Font.PLAIN));
				}
			}
		});

		optSyncOnUser = new JRadioButton("On user demand");
		if (UserPreferences.getCurrentInstance().whenAllRefreshing() == UserPreferences.ON_USER_DEMAND){
			optSyncOnUser.setSelected(true);
			optSyncOnUser.setFont(optSyncOnUser.getFont().deriveFont(Font.BOLD));
		}else
			optSyncOnUser.setFont(optSyncOnUser.getFont().deriveFont(Font.PLAIN));
		optSyncOnUser.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().setRefreshAllOn(UserPreferences.ON_USER_DEMAND);
				if (optSyncOnUser.isSelected()){
					optSyncOnUser.setFont(optSyncOnUser.getFont().deriveFont(Font.BOLD));
					optSyncOnActivation.setFont(optSyncOnActivation.getFont().deriveFont(Font.PLAIN));
					optSyncOnAnyAladinModification.setFont(optSyncOnAnyAladinModification.getFont().deriveFont(Font.PLAIN));
				}
			}
		});

		ButtonGroup synchronizationGroup = new ButtonGroup();
		synchronizationGroup.add(optSyncOnAnyAladinModification);
		synchronizationGroup.add(optSyncOnActivation);
		synchronizationGroup.add(optSyncOnUser);

		JPanel panSynchronizationOn = new JPanel(new GridLayout(3,1));
		panSynchronizationOn.setBorder(BorderFactory.createTitledBorder("How synchronize SEDPlugin with Aladin ?"));
		panSynchronizationOn.add(optSyncOnAnyAladinModification);
		panSynchronizationOn.add(optSyncOnActivation);
		panSynchronizationOn.add(optSyncOnUser);

		// Synchronization problem:
		/*optSyncPbm_Ask = new JRadioButton("ask");
		if (UserPreferences.getCurrentInstance().onSynchronizationProblemAction() == UserPreferences.ASK){
			optSyncPbm_Ask.setSelected(true);
			optSyncPbm_Ask.setFont(optSyncPbm_Ask.getFont().deriveFont(Font.BOLD));
		}else
			optSyncPbm_Ask.setFont(optSyncPbm_Ask.getFont().deriveFont(Font.PLAIN));
		optSyncPbm_Ask.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().setOnSynchronizationProblemAction(UserPreferences.ASK);
				if (optSyncPbm_Ask.isSelected()){
					optSyncPbm_Ask.setFont(optSyncPbm_Ask.getFont().deriveFont(Font.BOLD));
					optSyncPbm_DefaultAction.setFont(optSyncPbm_DefaultAction.getFont().deriveFont(Font.PLAIN));
					optSyncPbm_Nothing.setFont(optSyncPbm_Nothing.getFont().deriveFont(Font.PLAIN));
				}
			}
		});

		optSyncPbm_DefaultAction = new JRadioButton("do the corresponding action");
		optSyncPbm_DefaultAction.setToolTipText("<html><ul><li>If an image has been deleted then <b>remove</b> the associated point from all spectra</li>" +
										"<li>If a drawing has been deleted then <b>delete</b> its associated spectrum</li>" +
										"<li>If a drawing has been modified then <b>update</b> its associated spectrum.</li></ul></html>");
		if (UserPreferences.getCurrentInstance().onSynchronizationProblemAction() == UserPreferences.APPLY_ALL){
			optSyncPbm_DefaultAction.setSelected(true);
			optSyncPbm_DefaultAction.setFont(optSyncPbm_DefaultAction.getFont().deriveFont(Font.BOLD));
		}else
			optSyncPbm_DefaultAction.setFont(optSyncPbm_DefaultAction.getFont().deriveFont(Font.PLAIN));
		optSyncPbm_DefaultAction.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().setOnSynchronizationProblemAction(UserPreferences.APPLY_ALL);
				if (optSyncPbm_DefaultAction.isSelected()){
					optSyncPbm_DefaultAction.setFont(optSyncPbm_DefaultAction.getFont().deriveFont(Font.BOLD));
					optSyncPbm_Ask.setFont(optSyncPbm_Ask.getFont().deriveFont(Font.PLAIN));
					optSyncPbm_Nothing.setFont(optSyncPbm_Nothing.getFont().deriveFont(Font.PLAIN));
				}
			}
		});

		optSyncPbm_Nothing = new JRadioButton("do nothing");
		if (UserPreferences.getCurrentInstance().onSynchronizationProblemAction() == UserPreferences.IGNORE_ALL){
			optSyncPbm_Nothing.setSelected(true);
			optSyncPbm_Nothing.setFont(optSyncPbm_Nothing.getFont().deriveFont(Font.BOLD));
		}else
			optSyncPbm_Nothing.setFont(optSyncPbm_Nothing.getFont().deriveFont(Font.PLAIN));
		optSyncPbm_Nothing.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (e.getModifiers() == ActionEvent.MOUSE_EVENT_MASK)
					UserPreferences.getCurrentInstance().setOnSynchronizationProblemAction(UserPreferences.IGNORE_ALL);
				if (optSyncPbm_Nothing.isSelected()){
					optSyncPbm_Nothing.setFont(optSyncPbm_Nothing.getFont().deriveFont(Font.BOLD));
					optSyncPbm_Ask.setFont(optSyncPbm_Ask.getFont().deriveFont(Font.PLAIN));
					optSyncPbm_DefaultAction.setFont(optSyncPbm_DefaultAction.getFont().deriveFont(Font.PLAIN));
				}
			}
		});

		ButtonGroup syncPbmGroup = new ButtonGroup();
		syncPbmGroup.add(optSyncPbm_Ask);
		syncPbmGroup.add(optSyncPbm_DefaultAction);
		syncPbmGroup.add(optSyncPbm_Nothing);

		JPanel panSyncPbm = new JPanel(new GridLayout(3,1));
//		panSyncPbm.setLayout(new BoxLayout(panSyncPbm, BoxLayout.Y_AXIS));
		panSyncPbm.setBorder(BorderFactory.createTitledBorder("On Aladin stack modification..."));
		optSyncPbm_Ask.setAlignmentX(Component.LEFT_ALIGNMENT);
		panSyncPbm.add(optSyncPbm_Ask);
		optSyncPbm_DefaultAction.setAlignmentX(Component.LEFT_ALIGNMENT);
		panSyncPbm.add(optSyncPbm_DefaultAction);
		optSyncPbm_Nothing.setAlignmentX(Component.LEFT_ALIGNMENT);
		panSyncPbm.add(optSyncPbm_Nothing);*/

		panSynchronization.add(panSynchronizationOn);
		//panSynchronization.add(panSyncPbm);

		return panSynchronization;
	}

	/* **** */
	/* MAIN */
	/* **** */
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		(new PreferencesDialog(null)).setVisible(true);
	}

}
