package sedplugin.interfaceGraphique;


import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.URL;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.SwingWorker.StateValue;
import javax.swing.border.LineBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import sedplugin.graphe.Serie;
import sedplugin.plugin.SEDPlugin;
import sedplugin.sed.SED;
import sedplugin.sed.source.SEDSource;
import sedplugin.sed.source.aladin.Circle;
import sedplugin.sed.source.aladin.Polygon;
import sedplugin.sed.source.operation.Operation;

public class SEDJListItem extends JPanel implements PropertyChangeListener, ListSelectionListener {
	private static final long serialVersionUID = 1L;

	public static final int HEIGHT = 20;

	public final SEDJList parent;

	/** The check box to tell whether the Serie must be drawn. */
	private JCheckBox chkDrawn;

	/** The panel which lets switch between the display and the editor of the SED name. */
	private JPanel cardsName;

	/** The label which will print the name of the serie and its type. */
	private JLabel lblName;

	/** The text area to change the name of the SED. */
	private JTextArea txtName;

	/** The progress bar which will print a spectrum loading progression if the cell value is a Double or Object[]{Double,String}. */
	private JProgressBar progress;

	private final ItemContentMouseListener mouseListener;

	private final ItemContentKeyListener keyListener;

	/** The associated serie. */
	private final Serie serie;

	private int index = 0;

	private boolean selected = false;

	private boolean current = false;

	/** Icon for a spectrum coming from a position selection. */
	private static ImageIcon icoPos = null;

	/** Icon for a spectrum coming from a circle selection. */
	private static ImageIcon icoCircle = null;

	/** Icon for a spectrum coming from a polygon selection. */
	private static ImageIcon icoPolygon = null;

	/** Icon for a spectrum coming from a spectrums operation. */
	private static ImageIcon icoOperation = null;

	/** Icon for a spectrum coming from a catalog. */
	private static ImageIcon icoCatalog = null;

	/* ************ */
	/* CONSTRUCTORS */
	/* ************ */
	/**
	 * Initialize the CellRenderer.
	 */
	public SEDJListItem(final SEDJList parent, final Serie s, final int index){
		super();

		setBackground(UIManager.getColor("List.background"));
		setMinimumSize(new Dimension(100, HEIGHT));
		setMaximumSize(new Dimension(Integer.MAX_VALUE, HEIGHT));

		this.parent = parent;
		this.index = index;
		serie = s;

		mouseListener = new ItemContentMouseListener(this);
		keyListener = new ItemContentKeyListener(this);

		// Initialize the JCheckBox component:
		chkDrawn = new JCheckBox();
		chkDrawn.setOpaque(false);
		chkDrawn.addMouseListener(mouseListener);
		chkDrawn.addKeyListener(keyListener);
		chkDrawn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				updateGraph();
			}
		});

		// Initialize the JLabel component:
		lblName = new JLabel();
		lblName.setFocusable(true);
		lblName.setRequestFocusEnabled(true);
		lblName.addKeyListener(keyListener);
		lblName.addMouseListener(mouseListener);
		lblName.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent evt) {
				lblName.requestFocus();
				if (evt.getClickCount() > 1)
					triggerRename();
			}
		});

		// Initialize the JTextArea component:
		txtName = new JTextArea();
		txtName.addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent evt) {
				if (evt.getKeyCode() == KeyEvent.VK_ENTER){
					serie.getSED().setName(txtName.getText());
					((CardLayout)cardsName.getLayout()).show(cardsName, "DISPLAY");
					update();
				}else if (evt.getKeyCode() == KeyEvent.VK_ESCAPE)
					((CardLayout)cardsName.getLayout()).show(cardsName, "DISPLAY");
			}
		});
		txtName.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent evt) {
				((CardLayout)cardsName.getLayout()).show(cardsName, "DISPLAY");
			}
		});

		// Initialize the JProgressBar component:
		progress = new JProgressBar(0,100);
		progress.setStringPainted(true);
		progress.addMouseListener(mouseListener);
		progress.addKeyListener(keyListener);

		// Load all required icons:
		if (icoPos == null){
			URL urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"position.gif");
			if (urlIcon != null)
				icoPos = new ImageIcon(urlIcon);
		}
		if (icoCircle == null){
			URL urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"circle.gif");
			if (urlIcon != null)
				icoCircle = new ImageIcon(urlIcon);
		}
		if (icoPolygon == null){
			URL urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"polygon.gif");
			if (urlIcon != null)
				icoPolygon = new ImageIcon(urlIcon);
		}
		if (icoOperation == null){
			URL urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"calculator.gif");
			if (urlIcon != null)
				icoOperation = new ImageIcon(urlIcon);
		}
		if (icoCatalog == null){
			URL urlIcon = getClass().getResource(SEDPlugin.ICONS_DIR+"source3.png");
			if (urlIcon != null)
				icoCatalog = new ImageIcon(urlIcon);
		}

		setLayout(new CardLayout());
		JPanel pName = new JPanel(new BorderLayout());
		pName.setOpaque(false);
		cardsName = new JPanel(new CardLayout());
		cardsName.setOpaque(false);
		cardsName.add(lblName, "DISPLAY");
		cardsName.add(txtName, "EDITOR");
		pName.add(cardsName, BorderLayout.CENTER);
		pName.add(chkDrawn, BorderLayout.WEST);
		add(pName, "NAME");
		JPanel pProgress = new JPanel(new BorderLayout());
		pProgress.setOpaque(false);
		pProgress.add(progress, BorderLayout.CENTER);
		add(pProgress, "PROGRESS");

		update();
	}

	protected final void updateGraph(){
		Vector<Serie> series;
		if (chkDrawn.isSelected())
			series = serie.getGraph().addSerie(serie);
		else
			series = serie.getGraph().removeSerie(serie);
		parent.drawOnly(series);
		update();
	}

	public final int getIndex(){
		return index;
	}

	public final void setIndex(final int index){
		this.index = index;
	}

	public final Serie getSerie(){
		return serie;
	}

	public void setChecked(boolean checked) {
		chkDrawn.setSelected(checked);
	}

	public void triggerRename() {
		txtName.setText(serie.getSED().getName());
		((CardLayout)cardsName.getLayout()).show(cardsName, "EDITOR");
		txtName.requestFocus(true);
		txtName.selectAll();
	}

	/* ***************** */
	/* INHERITED METHODS */
	/* ***************** */
	public void update() {
		SED sed = serie.getSED();
		Color color = serie.getColor();

		// set an icon according to the spectrum origin:
		if (sed.getSource() != null){
			SEDSource src = sed.getSource();
			String srcType = src.getType();
			if (srcType.equals(Circle.CIRCLE_TYPE)){
				lblName.setIcon(icoCircle);
				color = ((Circle)src).getColor();
			}else if (srcType.equals(Polygon.POLYGON_TYPE)){
				lblName.setIcon(icoPolygon);
				color = ((Polygon)src).getColor();
			}else if (srcType.equals(Operation.OPERATION_TYPE))
				lblName.setIcon(icoOperation);
			else
				lblName.setIcon(null);
		}

		if (current){
			setBorder(new DottedBorder(UIManager.getColor("List.selectionBackground").brighter()));
		}else
			setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));

		/* set the text color to the color of the associated serie
		 * if currently drawn: */
		if (chkDrawn.isSelected())
			lblName.setForeground(color);
		else
			lblName.setForeground(UIManager.getColor("List.foreground"));

		// set the background in function of the selection state of this item:
		if (selected){
			lblName.requestFocus();
			setBackground(UIManager.getColor("List.selectionBackground"));
		}else
			setBackground(UIManager.getColor("List.background"));

		// set the spectrum name in bold characters if the spectrum is frozen/constant:
		if (sed.getSource() == null || sed.getSource().isFrozen())
			lblName.setFont(getFont().deriveFont(Font.ITALIC));
		else
			lblName.setFont(getFont().deriveFont(Font.PLAIN));
		// print the spectrum name:
		lblName.setText(sed.getName());
		lblName.setToolTipText(sed.getName());
	}

	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals("state")){
			StateValue state = (StateValue)evt.getNewValue();
			switch(state){
			case PENDING:
				((CardLayout)this.getLayout()).show(this, "PROGRESS");
				progress.setIndeterminate(true);
				break;
			case STARTED:
				((CardLayout)this.getLayout()).show(this, "PROGRESS");
				progress.setValue(0);
				break;
			case DONE:
				progress.setValue(100);
				if (serie != null && serie.getSED() != null && !serie.getSED().isEmpty()){
					((CardLayout)this.getLayout()).show(this, "NAME");
					serie.build();
					if (serie.getGraph().getNbSeries() == 0){
						chkDrawn.setSelected(true);
						updateGraph();
					}else if (chkDrawn.isSelected())
						serie.getGraph().update();
					if (selected)
						parent.fireContentsChanged(index);
				}
				break;
			}
		}else if (evt.getPropertyName().equals("progress"))
			progress.setValue(((Integer)evt.getNewValue()).intValue());
	}

	private final class ItemContentMouseListener implements MouseListener {
		private final SEDJListItem realSource;
		public ItemContentMouseListener(final SEDJListItem item){
			realSource = item;
		}
		private MouseEvent modifyEvent(MouseEvent evt){
			evt.setSource(realSource);
			return evt;
		}
		public void mouseClicked(MouseEvent evt) {
			processMouseEvent(modifyEvent(evt));
		}
		public void mouseEntered(MouseEvent evt) {
			processMouseEvent(modifyEvent(evt));
		}
		public void mouseExited(MouseEvent evt) {
			processMouseEvent(modifyEvent(evt));
		}
		public void mousePressed(MouseEvent evt) {
			processMouseEvent(modifyEvent(evt));
		}
		public void mouseReleased(MouseEvent evt) {
			processMouseEvent(modifyEvent(evt));
		}
	}

	private final class ItemContentKeyListener implements KeyListener {
		private final SEDJListItem realSource;
		public ItemContentKeyListener(final SEDJListItem item){
			realSource = item;
		}
		private KeyEvent modifyEvent(KeyEvent evt){
			evt.setSource(realSource);
			return evt;
		}
		private boolean noModifier(KeyEvent evt){
			return !evt.isAltDown() && !evt.isAltGraphDown() && !evt.isControlDown() && !evt.isMetaDown() && !evt.isShiftDown();
		}
		public void keyPressed(KeyEvent evt) {
			if (noModifier(evt)){
				if (evt.getKeyCode() == KeyEvent.VK_F2){
					realSource.txtName.setText(realSource.serie.getSED().getName());
					((CardLayout)realSource.cardsName.getLayout()).show(realSource.cardsName, "EDITOR");
					realSource.txtName.requestFocus(true);
					realSource.txtName.selectAll();
				}else if (evt.getKeyCode() == KeyEvent.VK_SPACE || evt.getKeyCode() == KeyEvent.VK_ENTER){
					realSource.chkDrawn.setSelected(!realSource.chkDrawn.isSelected());
					realSource.updateGraph();
				}
			}
			processKeyEvent(modifyEvent(evt));
		}
		public void keyReleased(KeyEvent evt) {
			processKeyEvent(modifyEvent(evt));
		}
		public void keyTyped(KeyEvent evt) {
			processKeyEvent(modifyEvent(evt));
		}
	}

	public void valueChanged(ListSelectionEvent evt) {
		if (!evt.getValueIsAdjusting()){
			if (index >= evt.getFirstIndex() && index <= evt.getLastIndex()){
				if (parent.isSelectedIndex(index) != selected){
					selected = !selected;
					current = parent.isCurrent(index);
					update();
				}else if (parent.isCurrent(index) != current){
					current = !current;
					update();
				}
			}
		}
	}

	private final class DottedBorder extends LineBorder {
		private static final long serialVersionUID = 1L;

		public DottedBorder(Color color) {
			super(color);
		}
		@Override
		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
			Graphics2D g2 = ((Graphics2D)g);
			g2.setColor(lineColor);
			g2.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 1.0f, new float[]{1.0f}, 0.0f));
			g2.drawRect(x, y, width-1, height-1);
		}
	}

}
