package sedplugin.sed.source.operation;

import java.util.Stack;

public class NPI {

	public static String toNPI(final String expr) throws Exception {
		Stack<String> operands = new Stack<String>();
		Stack<String> operators = new Stack<String>();

		String[] tokens = expr.split("[ \t\n]");

		for(String token : tokens){
			if (token.equals("+") || token.equals("-") || token.equals("*") || token.equals("/") || token.equals("^") || token.equals("(")){
				if (!operators.isEmpty() && !hasPriority(token) && hasPriority(operators.peek()))
					unstack(operands, operators);
				operators.push(token);
			}else if (token.equals(")")){
				unstackAll(operands, operators);
				if (!operators.isEmpty() && hasPriority(operators.peek()))
					unstack(operands, operators);
			}else
				operands.push(token);
		}

		return unstackAll(operands, operators);
	}

	private static boolean hasPriority(final String operator){
		return (operator != null && (operator.equals("*") || operator.equals("/") || operator.equals("^")));
	}

	private static String unstack(Stack<String> operands, Stack<String> operators) throws Exception {
		StringBuffer buf = new StringBuffer();

		buf.append(operands.pop()).append(operands.pop()).append(operators.pop());
		operands.push(buf.toString());

		if (operands.isEmpty() && !operators.isEmpty())
			throw new Exception("ERROR: Too much operators !");

		return operands.peek();
	}

	private static String unstackAll(Stack<String> operands, Stack<String> operators) throws Exception {
		StringBuffer buf = new StringBuffer();

		String operator;
		while(!operators.isEmpty()){
			operator = operators.pop();
			if (operator.equals("("))
				return buf.toString();
			else{
				buf.append(operands.pop()).append(operands.pop()).append(operator);
				operands.push(buf.toString());
				buf.delete(0, buf.length());
			}
		}

		if (operands.isEmpty() && !operators.isEmpty())
			throw new Exception("ERROR: Too much operators !");

		return operands.pop();
	}

	/* ************** */
	/* TODO SELF TEST */
	/* ************** */
	public static void main(String[] args) throws Exception {
		String toParse = "( 1 + 2 ) * 4 + 5 - $1";
		System.out.println("BEFORE: ["+toParse+"]");
		System.out.println("AFTER : ["+NPI.toNPI(toParse)+"]");
	}

}
